package com.zhiLian.datasource.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.zhiLian.datasource.DataSourceInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author DongZl
 * @description: 数据源配置
 * @Date 2023-8-1 上午 11:05
 */
@Log4j2
@Configuration
public class DruidConfig {


    private List<DataSourceInfo> getDataSourceInfoList(){
        List<String> databaseNameList = new ArrayList<>(){{
            add("3306");
//            add("3307");
        }};
        return databaseNameList.stream().map(DataSourceInfo::databaseNameBuild).toList();
    }

    /**
     * @Description: 根据传递的数据源信息测试数据库连接
     * @Author Dongzl
     */
    public DruidDataSource createDataSourceConnection(DataSourceInfo dataSourceInfo) {
        DruidDataSource druidDataSource = new DruidDataSource();
        druidDataSource.setUrl(dataSourceInfo.getUrl());
        druidDataSource.setUsername(dataSourceInfo.getUserName());
        druidDataSource.setPassword(dataSourceInfo.getPassword());
        druidDataSource.setBreakAfterAcquireFailure(true);
        druidDataSource.setConnectionErrorRetryAttempts(0);
        try {
            druidDataSource.getConnection(2000);
            log.info("{} -> 数据源连接成功", dataSourceInfo.getKey());
            return druidDataSource;
        } catch (SQLException throwables) {
            log.error("数据源 {} 连接失败,用户名：{}，密码 {}",dataSourceInfo.getUrl(),dataSourceInfo.getUserName(),dataSourceInfo.getPassword());
            return null;
        }
    }

    @Bean
    @Primary
    public DynamicDataSource dynamicDataSource() {

        Map<Object, Object> dataSourceMap = new HashMap<>();
        getDataSourceInfoList().forEach(dataSourceInfo -> {
            dataSourceMap.put(dataSourceInfo.getKey(), createDataSourceConnection(dataSourceInfo));
        });
        //设置动态数据源
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
//        dynamicDataSource.setDefaultTargetDataSource(masterDataSource());
        dynamicDataSource.setTargetDataSources(dataSourceMap);
        //将数据源信息备份在defineTargetDataSources中
        dynamicDataSource.setDefineTargetDataSources(dataSourceMap);
        return dynamicDataSource;
    }
}
